// OK    LCA  cu RMQ (O(N*log N), O(1))   (preprocessing, query)

#include <stdio.h>
#include <math.h>
#define DIM 100
#define __DEBUG__

#define IN "lca.in"
#define OUT "lca.out"


int a[DIM][17];     // a[i][j] - poz minimului pt sirul niv in intervalul [i, i + 2^j - 1]
int niv[2*DIM];     // nivelele varfurilor din Turul Euler in arbore niv[i] == nivelul[E[i]]
int contor;
int E[2*DIM];       // Turul Euler
int Poz[DIM];       // Poz[i] - poz primei aparitii in Tur a lui i

int l[DIM];
int n, q, k, i, j, x, y;

typedef struct nod {
	int vf;
	nod* urm;
} *PNOD, NOD;
int s[DIM];      // selectat
PNOD L[DIM];

void Read();
void Add( int i, int j);
void DF(int, int);
void RMQ();
void LCA();

int main()
{
	Read();
	DF(10, 1);     // 10 -radacina ( in fisierul meu)


#if defined __DEBUG__
	// debug
	printf("E   ");
	for ( i = 1; i <= 2*n - 1; i++ )
		printf("%d ", E[i]);
	printf("\nniv  ");

	for ( i = 1; i <= 2*n - 1; i++ )
		printf("%d ", niv[i]);

	printf("\nPoz  ");
	for ( i = 1; i <= n; i++ )
		printf("%d ", Poz[i]);
	printf("\n");
#endif

	n = 2*n - 1;     // niv[] si E[] au 2*n - 1 elemente
	RMQ();
	LCA();

	fclose(stdin);
	fclose(stdout);

  return 0;
}

void RMQ()  // x, y pozitii
{
/*       si asa e OK cauta k max astfel incat 2^k <= n
  k = 0;
  while (1 << k <= n)  k++;
  k--;
*/
	k = (int)floor(log(n)/log(2));     // k == log2(n)

	for ( i = 1; i <= n; i++ ) a[i][0] = i;   // indicii sirului niv se pun pe prima coloana
                                           // a[i][0] = pozitia minimului pt sirul niv[]
	for ( j = 1; j <= k; j++ )             // in intervalul [i, i + 2^0 - 1] = i
		for ( i = 1; i <= n; i++ )    // pe coloane !
		{
			a[i][j] = a[i][j-1];     // poz minimului in interv [i, i + 2^(j-1) -1]
			if (i + (1 << (j-1)) <= n && niv[a[i][j]] > niv[a[i+(1 << (j-1))][j-1]] )
				a[i][j] = a[i+(1 << (j-1))][j-1]; // a[i+(1<<(j-1))][j-1]]  e poz minimului pt sirul niv[] in inter-
		}                                         // valul [i+2^(j-1), 1 + 2^j -1]

#if !defined __DEBUG__
	for ( i = 1; i <= k; i++ )
	{
		for ( j = 0; j <= k; j++ )
			printf("%4d", a[i][j]);
		printf("\n");
	}
	printf("\n\n\n");
#endif
/*
  l[1] = 0;     // log2(1)
  for ( i = 2; i <= n; i++ )l[i] = 1 + l[i >> 1];    // calculeaza log2(i) - nu e necesar
*/
}                                                    // mai jos in  LCA() am folosit formula pt log2(i)


void LCA()
{
	int v1 = 0, v2 = 0;
	for ( i = 1; i <= q; i++ ) // pt fiecare query
	{
		scanf("%d %d", &v1, &v2);
		x = Poz[v1];       // x, y pozitii in sirul E
		y = Poz[v2];
		if ( x > y )
		{
			int aux = x; x = y; y = aux;
		}
		if (x == y)
		{
			printf("%d\n", E[a[x][0]]); // nodul din E[] care coresp pozitiei minimului in niv[]
			continue;
		}


//	    k = l[y-x];
		k = (int)floor(log(y-x)/log(2)); // log2(a) = ln(a)/ln(2)

		if (niv[a[x][k]] <= niv[a[y-(1 << k)+1][k]] )
			printf("%d\n", E[a[x][k]]);
		else
			printf("%d\n", E[a[y-(1 << k)+1][k]]);
	}
}

void DF(int varf, int nivel)
{
	s[varf] = 1;
	E[++contor] = varf;
	Poz[varf] = contor;    //
	niv[contor] = nivel;
	for ( PNOD p = L[varf]; p; p = p->urm )
		if ( !s[p->vf] )
		{
			DF(p->vf, nivel + 1);
			E[++contor] = varf;
			niv[contor] = nivel;
		}
}

void Add( int i, int j)
{
	PNOD p = new NOD;
	p->vf = j;
	p->urm = L[i];
	L[i] = p;
}

void Read()
{
	freopen(IN, "r", stdin);
	freopen(OUT, "w", stdout);
	scanf("%d %d", &n, &q);   // noduri, query

	int v1 = 0, v2 = 0;
	for ( int i = 1; i <= n - 1; i++ )
	{
		scanf("%d %d", &v1, &v2);
		Add(v1, v2);
		Add(v2, v1);
	}
}

